import {
  select,
  csv, 
  scaleLinear, 
  max, 
  scaleBand, 
  axisLeft,
  axisBottom,
  format
} from 'd3';
import { getBarData } from './data';

// Get the SVG from the DOM
const svg = select('svg');

// Extract Width and Height from SVG and parse it to float
const height = +svg.attr('height');
const width  = +svg.attr('width');
const margin = {top: 55, right: 45, bottom: 80, left: 195};
const innerWidth = width-margin.left-margin.right;
const innerHeight = height-margin.top-margin.bottom;

const render = data => {
  const xValue = d => d.population;
  const yValue = d => d.country;

  // Set Scales of X and Y Axis
  const xScale = scaleLinear()
  	.domain([0,max(data, xValue)])
  	.range([0,innerWidth]);
  const yScale = scaleBand()
  	.domain(data.map(yValue))
  	.range([0,innerHeight])
  	.padding(0.1);
  
  // Format X Axis
  const xAxisTickFormat = number =>
  	format('.3s')(number)
  		.replace('G','B');
  const xAxis = axisBottom(xScale)
  	.tickFormat(xAxisTickFormat)
  	.tickSize(-innerHeight);

  // Add Main Bar Group
  const barGroup = svg
  	.append('g')
  		.attr('transform', `translate(${margin.left}, ${margin.top})`)
  		.attr('text-anchor','middle');

  // Add Y Axis Group
  const yAxisGroup = barGroup
    .append('g')
    	.call(axisLeft(yScale));
	yAxisGroup
  	.selectAll('.domain, .tick line')
  		.remove();

  // Add X Axis Group
  const xAxisGroup = barGroup
    .append('g')
    	.call(xAxis)
  		.attr('transform', `translate(0, ${innerHeight})`);
  xAxisGroup
    .select('.domain')
  		.remove();
  xAxisGroup
  	.append('text')
  		.attr('class','axisLabel')
  		.attr('x',innerWidth/2)
  		.attr('y',65)
  		.attr('fill', 'black')
  		.text('Population');

  // Add The Bars
  barGroup
    .selectAll('rect')
  	.data(data)
  		.enter().append('rect')
  			.attr('y', d => yScale(yValue(d)))
  			.attr('width', d => xScale(xValue(d)))
  			.attr('height',yScale.bandwidth());

  // Add The Title
  barGroup.append('text')
		.attr('class','titleLabel')
  	.attr('x',innerWidth/2)
  	.attr('y', -10)
  	.text('Population by Country');

};

getBarData()
  .then(data => {
  	render(data);
});

